%option noyywrap
%option yylineno

%{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "y.tab.h"
#include "raichu_alu.h"
#include "raichu_fpu.h"

unsigned int commentlv = 0;
%}

%state COMMENT
SPACE   ("\n"|"\r"|"\t"|" ")+
DIGIT   [0-9]
LOWER   [a-z]
UPPER   [A-Z]

%%

<INITIAL>{SPACE}+  ;
<INITIAL>\(        {return LPAREN;}
<INITIAL>\)        {return RPAREN;}
<INITIAL>"true"    {
	yylval.val = 0xffffffffU;
	return BOOL_VAL;}
<INITIAL>"false"   {
	yylval.val = 0x00000000U;
	return BOOL_VAL;}
<INITIAL>{DIGIT}+  {
	yylval.val = alu_atoi(yytext);
	return INT_VAL;}
<INITIAL>{DIGIT}+(\.{DIGIT}*)?([eE][\+\-]?{DIGIT}+)? {
	yylval.val = fpu_atof(yytext);
	return FLOAT_VAL;}
<INITIAL>"-"       {return MINUS;}
<INITIAL>"="       {return EQUAL;}
<INITIAL>"let"     {return LET;}
<INITIAL>"in"      {return IN;}
<INITIAL>","       {return COMMA;}
<INITIAL>"create_array" {return ARRAY_CREATE;}
<INITIAL>"."       {return DOT;}
<INITIAL>{LOWER}(({DIGIT}|{LOWER}|{UPPER}|"_")*) {
	yylval.name = (char*)malloc(strlen(yytext)+1);
	if(!(yylval.name)){
		fprintf(stderr, "malloc failed\n");
		exit(1);
	}
	strcpy(yylval.name, yytext);
	return IDENT;}

<INITIAL>\(\*      {commentlv = 1; BEGIN COMMENT;}

<INITIAL>.         {fprintf(stderr, "unknown character[%s]\n", yytext);}

<COMMENT>\(\*      {commentlv++;}
<COMMENT>.         ;
<COMMENT>\n        ;
<COMMENT>\*\)      {commentlv--; if(!commentlv){BEGIN INITIAL;}}

%%
